
def minDis(s1, s2, n, m, dp) :
    '''This is a memoized version of recursion i.e. Top-Down DP: to find minimum number
    operations to convert str1 to str2
    external credit: # This code is contributed by divyesh072019.
    source: GeeksforGeeks https://www.geeksforgeeks.org/edit-distance-dp-5/
    '''
    # If any string is empty,
    # return the remaining characters of other string         
    if(n == 0) :
        return m       
    if(m == 0) :
        return n
                        
    # To check if the recursive tree
    # for given n & m has already been executed
    if(dp[n][m] != -1)  :
        return dp[n][m];
                    
    # If characters are equal, execute
    # recursive function for n-1, m-1   
    if(s1[n - 1] == s2[m - 1]) :          
        if(dp[n - 1][m - 1] == -1) :
            dp[n][m] = minDis(s1, s2, n - 1, m - 1, dp)
            return dp[n][m]                  
        else :
            dp[n][m] = dp[n - 1][m - 1]
            return dp[n][m]
            
    # If characters are nt equal, we need to          
    # find the minimum cost out of all 3 operations.        
    else :           
        if(dp[n - 1][m] != -1) :  
            m1 = dp[n - 1][m]     
        else :
            m1 = minDis(s1, s2, n - 1, m, dp)
                
        if(dp[n][m - 1] != -1) :               
            m2 = dp[n][m - 1]           
        else :
            m2 = minDis(s1, s2, n, m - 1, dp)  
        if(dp[n - 1][m - 1] != -1) :   
            m3 = dp[n - 1][m - 1]   
        else :
            m3 = minDis(s1, s2, n - 1, m - 1, dp)
        
        dp[n][m] = 1 + min(m1, min(m2, m3))
        return dp[n][m]
     

 


def Levenshtein(str1, str2):
    '''this is the Levenshtein similarity ratio 
    https://www.datacamp.com/tutorial/fuzzy-string-python
    '''
        
    n = len(str1)
    m = len(str2)
    dp = [[-1 for i in range(m + 1)] for j in range(n + 1)]
                
    lev=minDis(str1, str2, n, m, dp)

    return  ((n + m) - lev) / (n+m)




import requests 
import json 

import numpy as np
import pandas

from tqdm import tqdm
from time import sleep

import matplotlib.pyplot as plt 
from bcrpy.anexo import Levenshtein


class Marco:
    def __init__(self):
        '''Este es el marco principal para almacenar variables y ejecutar metodos para extraer, buscar y manejar datos. 

        Parametros
        ----------
        metadata: <Pandas DataFrame>
            Los metadatos de las series estadísticas del BRCPData, los cuales pueden ser reducidos con el metodo ref_metadata de esta `class`. 
        codigos : list(str)
            lista de codigos de series en interes para usar con los metodos de esta `class`. 
        formato : str
            formato para extraer / procesar datos (predeterminado: json)
        fechaini : str
            fecha de inicio para la(s) serie(s) seleccionada(s) en mes año (A) y mes (M) (formato AAAA-M)
        fechafin : str
            fecha de final para la(s) serie(s) seleccionada(s) en mes año (A) y mes (M) (formato AAAA-M)
        idioma : str
            idioma seleccionado (predeterminado: ing) otra opcion es 'esp'
        '''
        self.metadata = ''
        self.codigos = ['PN01288PM','PN01289PM']
        self.formato = 'json'
        self.fechaini = '2010-1'
        self.fechafin = '2016-9'
        self.idioma = 'ing'

    def state_inputs(self):
        '''Declara el estado actual de todas las variables constructoras de la clase Marco. 
        '''

        print('''corriendo estado actual de todas las variables constructoras...\n
self.metadata = {}
self.codigos = {}
self.formato = {}
self.fechaini = {}
self.fechafin = {}
self.idioma = {}
'''.format('<vacio>' if len(self.metadata)==0 else str(type(self.metadata))+' size: '+str(self.metadata.shape),
        self.codigos,
        self.formato,
        self.fechaini, 
        self.fechafin,
        self.idioma))


    def get_metadata(self, filename='metadata.csv'): 
        '''Extrae todos los metadatos de BCRPData. 
        
        Parametros
        ----------
        filename : str
            Nombre del archivo para guardar todos los metadatos extraidos como un archivo .csv (predeterminado: 'metadata.csv'). Si se desea no guardar un archivo, cambiar a filename=''
        '''
        self.metadata = pandas.read_csv(
            'https://estadisticas.bcrp.gob.pe/estadisticas/series/metadata', delimiter=';', encoding='latin-1'
            )
        
        if filename[-3:] == 'csv':
            self.metadata.to_csv(filename,sep=";",index=False, index_label=False)

    def load_metadata(self,filename='metadata.csv'):
        '''Carga los metadatos guardados como archivo .csv a Python.

        Parametros
        ----------
        filename : str
            Nombre del archivo .csv del cual cargar los metadatos a Python.
        '''

        if filename[-3:] == 'csv':
            self.metadata = pandas.read_csv(filename, delimiter=';')

    def save_metadata(self,filename='metadata_new.csv'):
        '''Guarda los metadatos de self.metadata como archivo .csv

        Parametros
        ----------
        filename : str
            Nombre para el archivo .csv (predeterminado = 'metadata_new.csv')
        '''
        if filename[-3:] == 'csv':
            self.metadata.to_csv(filename,sep=";",index=False, index_label=False)

        
    def query(self,codigo='PD39793AM'):
        '''Consulta (query) de codigo de serie, impresa en formato json. 

        Parametros
        ----------
        codigo : str
            Nombre de codigo de series a consultar
        '''
        self.get_metadata() if len(self.metadata) == 0 else None
        df = self.metadata


        print('corriendo query para {}...\n'.format(codigo))

        index = df.index[df.iloc[:,0] == codigo].tolist()
        print('{} es indice {} en metadatos'.format(codigo, index[0]))

        dict = self.metadata.loc[index].to_dict()
        for i in dict.keys():
            dict[i] = dict[i][index[0]]
        dict.pop("Unnamed: 13" ) if "Unnamed: 13" in dict.keys() else None

        jsondata = json.dumps(dict,indent = 8,ensure_ascii=False)
        print(jsondata)
        return jsondata



    def wordsearch(self,keyword='economia',fidelity=0.65,columnas='all',verbose=False):
        '''Busqueda difusa de palabra clave (keyword) en metadatos de BCRPData. Regresa una tabla de 
        datos en formato <Pandas DataFrame> de los metadatos asociados con aquella palabra. 

        Parametros
        ----------
        keyword : str
            Palabra clave para reducir los metadatos
        fidelity : float
            Este es el Levenshtein similarity ratio (predeterminado=0.65). Un fidelity de 1.00 solo regresara metadatos que contienen palabras que coinciden con la palabra clave al 100%.
        columnas : str
            Indices de columnas de los metadatos seleccionados para correr el metodo. Predeterminado='all' corre el metodo en todas las columnas. 
        verbose : bool
            Muestra las columnas que estan siendo elegidas mientras el metodo corre (predeterminado=False)
        '''

        print('corriendo wordsearch: `{}` (fidelity = {})* '.format(keyword,fidelity))
        print('*medido con Levenshtein similarity ratio')
        print('por favor esperar...\n')

        INDICES = []
        df=  self.get_metadata() if len(self.metadata) == 0 else self.metadata

        loop_range = range(12) if columnas == 'all' else columnas
        for k in tqdm(loop_range):
            sim_words = []
            for j in df.iloc[:,k]:
                wordstring = str(j)
                a = [Levenshtein(word,keyword) for word in wordstring.split()]
                b = [(word,keyword) for word in wordstring.split()]
                if (np.array(a) >= fidelity).any():
                    sim_words.append(wordstring)
            
            sim_words=list(set(sim_words))
            
            print(sim_words) if verbose else None

            sleep(.1)

            print()
            if len(sim_words) !=0:
                INDICES.extend(list(set(df.index[ df.iloc[:,k].isin(sim_words)].tolist())))

        INDICES = list(set(INDICES))
        new_df = df.iloc[INDICES]
        print('\n\n',new_df)
        return new_df

    def ref_metadata(self,filename=False):
        '''Reduce los metadatos en self.metadata a aquellos perteneciendo a los codigos de serie declarados en self.codigos. 

        Parametros
        ----------
        filename : str (opcional)
            Nombre para guardar la informacion de la modificada self.metadata como un archivo .csv 
        '''
                
        df=self.metadata
        indices = [ df.index[df.iloc[:,0] == k].tolist()[0] for k in self.codigos ]
        
        self.metadata = self.metadata.loc[indices]

        if filename:
            self.save_metadata(filename)


    def GET(self,filename=False):
        '''Extrae los datos del BCRPData selecionados por las previamente-declaradas variables `self.codigos`, `self.fechaini`, `self.fechafin`, `self.formato`, y `self.idioma`. 

        Parametros
        ----------
        filename : str (opcional)
            Nombre para guardar los datos extraidos como un archivo .csv
        '''

        root = 'https://estadisticas.bcrp.gob.pe/estadisticas/series/api'
        format = self.formato 
        code_series = '-'.join(self.codigos)
        period = '{}/{}'.format(self.fechaini,self.fechafin) 
        language = self.idioma

        url = "{}/{}/{}/{}/{}".format(root,code_series,format,period,language)
        print(url)

        'https://www.geeksforgeeks.org/response-json-python-requests/'
        response = requests.get(url)
        dict  = response.json()

        header = [k['name'] for k in dict['config']['series'] ]
        df = pandas.DataFrame(columns=header) # Note that there are now row data inserted.

        for j in dict['periods']:
            # print(j['values'])
            df.loc[j['name']] = [float(ij) if ij!='n.d.' else None for ij in j['values'] ]

        if filename:
            df.to_csv(filename,sep=",")
        
        # print(df)
        return df

    def plot(self, data, title='', titlesize=9, func='plot'):
        '''Grafica x-y data.

        Parametros
        ----------
        data : <Pandas DataFrame>
            Data x-y extraida de BCRPData, x es fecha y es cantidad. 
        title : str
            Titulo para grafica
        func : str
            Tipo de grafica. 'plot' es grafica comun, 'semilogy' es grafica con escala logaritmica en y-axis.  
        titlesize : str
            Tamaño de titulo para grafica
        '''
        plt.style.use("seaborn-pastel")

        plt.title(title, fontsize=titlesize)
        plt.grid(axis='y')
        eval('plt.{}(data)'.format(func))
        plt.xticks(data.index, rotation =60)
        plt.tight_layout()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bcrpy']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.7.1,<4.0.0',
 'numpy>=1.24.2,<2.0.0',
 'pandas>=1.5.3,<2.0.0',
 'requests>=2.28.2,<3.0.0',
 'tqdm>=4.65.0,<5.0.0']

setup_kwargs = {
    'name': 'bcrpy',
    'version': '0.1.1',
    'description': 'Un cliente API para la extraccion, consulta y analisis de la base de datos BCRPData del Banco Central de Reserva del Peru (BCRP)',
    'long_description': '# bcrpy\n\nUn cliente API para la extraccion, consulta y analisis de la base de datos [BCRPData](https://estadisticas.bcrp.gob.pe/estadisticas/series/) del [Banco Central de Reserva del Peru (BCRP)](https://www.bcrp.gob.pe/) escrito para Python. Este cliente es un _wrapper_ de la [API para Desarrolladores](https://estadisticas.bcrp.gob.pe/estadisticas/series/ayuda/api) del BCRP.\n\n![](img/logo-bcrp.png)\n\n\n\n# Installation and Local Usage\n\nTo import this repository to your local environment and access it (on terminal):\n\n```ruby\nvirtualenv venv\nsource venv/bin/activate\npip install bcrpy\n```\n\n# Examples\n#### [In]\n```ruby\nimport bcrpy\n\nbanc = bcrpy.Marco()\t\t\t# cargar objeto\n\nbanc.get_metadata()\t\t\t# obtener todos los metadatos del BCRPData \nprint(banc.metadata)\t\t# imprimir metadatos obtenidos\n```\n\n#### [Out]\n| | Código de serie      |          Categoría de serie  | ... | Memo Unnamed: 13 |\n| -----------           | -----------           | -----------                |-----------  | -----------  |\n|0 |          PN00001MM |  Sociedades creadoras de depósito |  ... | NaN |        NaN |\n|1 |          PN00002MM |  Sociedades creadoras de depósito |  ... | NaN |        NaN |\n|2 |          PN00003MM |  Sociedades creadoras de depósito |  ... | NaN |        NaN |\n|3 |          PN00004MM |  Sociedades creadoras de depósito |  ... | NaN |        NaN |\n|4 |          PN00005MM |  Sociedades creadoras de depósito |  ... | NaN |        NaN |\n| ... |              ... |                             | ... | ... | ... |        ... |\n| 14853 |      PD39791AM  |      Expectativas Empresariales | ... | NaN |        NaN |\n| 14854  |     PD39792AM  |      Expectativas Empresariales | ... | NaN |        NaN |\n| 14855  |     PD39793AM  |      Expectativas Empresariales | ... | NaN |        NaN |\n| 14856  |     PD39794AM  |      Expectativas Empresariales | ... | NaN |        NaN |\n| 14857  |     PD39795AM  |      Expectativas Empresariales | ... | NaN |        NaN |\n\n#### [In]\n```ruby\nimport bcrpy\n\nbanc = bcrpy.Marco()\t\t\t# cargar objeto\n\n#hacer una consulta del codigo de serie predeterminado de este metodo (actualmente \'PD39793AM\') con el API del BCRPData\nbanc.query()\t\t\t\n\n#hacer otra consulta, pero para el codigo de serie \'PN00015MM\'\nbanc.query(\'PN00015MM\')\t\t\n```\n\n#### [Out] \n\nrunning query for PD39793AM...\n\nPD39793AM es indice 14855 en metadatos\n```json\n{\n        "Código de serie": "PD39793AM",\n        "Categoría de serie": "Expectativas Empresariales",\n        "Grupo de serie": "Expectativas empresariales sectoriales",\n        "Nombre de serie": "Índice de expectativas del sector a 12 meses - Servicios",\n        "Fuente": NaN,\n        "Frecuencia": "Mensual",\n        "Fecha de creación": "2023-02-28",\n        "Grupo de publicación": "Expectativas macroeconómicas y de ambiente empresarial",\n        "Área que publica": "Departamento de Indicadores de la Actividad Economía",\n        "Fecha de actualización": "2023-02-28",\n        "Fecha de inicio": "Abr-2010",\n        "Fecha de fin": "Sep-2022",\n        "Memo": NaN\n}\n```\nrunning query for PN00015MM...\n\nPN00015MM es indice 14 en metadatos\n```json\n{\n        "Código de serie": "PN00015MM",\n        "Categoría de serie": "Sociedades creadoras de depósito",\n        "Grupo de serie": "Cuentas monetarias de las sociedades creadoras de depósito",\n        "Nombre de serie": "Activos Internos Netos - Crédito al Sector Privado - ME (millones US$)",\n        "Fuente": "BCRP",\n        "Frecuencia": "Mensual",\n        "Fecha de creación": "2022-03-24",\n        "Grupo de publicación": "Sistema financiero y empresas bancarias y expectativas sobre condiciones crediticias",\n        "Área que publica": "Departamento de Estadísticas Monetarias",\n        "Fecha de actualización": "2023-02-24",\n        "Fecha de inicio": "Abr-1992",\n        "Fecha de fin": "Sep-2022",\n        "Memo": NaN\n}\n```\n\n#### [In]\n```ruby\nimport bcrpy\nimport matplotlib.pyplot as plt \n\nbanc = bcrpy.Marco()\t#cargar objeto\n\n#escoger los inputs de los datos que se desean extraer del BCRPData (otros datos como banc.idioma (=\'ing\') son predeterminados, pero tambien se pueden cambiar)\nbanc.codigos = [\'PN01288PM\',\'PN01289PM\',\'PN00015MM\']\nbanc.fechaini = \'2020-1\'\nbanc.fechafin = \'2023-1\'\n\nbanc.state_inputs()\t\t\t# mostrar el estado actual de los inputs escogidos \n\ndf = banc.GET(\'GET.csv\')\t# obtener informacion de los inputs escogidos (arriba) con el API del BCRP y guardarlas como un archivo con el nombre \'GET.csv\'\n\n#graficos (plots)\nfor name in df.columns:\n    plt.figure(figsize=(9, 4))\n    banc.plot(df[name],name,\'plot\')\n    plt.show()\nplt.show()\n```\n\n#### [Out]\n\n\nrunning current inputs state...\n\n        self.metadata = \\<vacio>\n        self.codigos = [\'PN01288PM\', \'PN01289PM\', \'PN00015MM\']\n        self.formato = json\n        self.fechaini = 2020-1\n        self.fechafin = 2023-1\n        self.idioma = ing\n\nhttps://estadisticas.bcrp.gob.pe/estadisticas/series/api/PN01288PM-PN01289PM-PN00015MM/json/2020-1/2023-1/ing\n\n![](img/Figure_1.png)\n![](img/Figure_2.png)\n![](img/Figure_3.png)\n\n',
    'author': 'andrewrgarcia',
    'author_email': 'garcia.gtr@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
